#!/usr/bin/env python
# -*- coding: utf-8 -*-
from cliente import BaseAPI


class Tribunais(BaseAPI):

    def __init__(self, token, url=BaseAPI.URL_DEFAULT):
        super(Tribunais, self).__init__(token, url)
        self.tribunais = None

    def _obtem_dados(self):
        resposta = self.executa_tribunal()
        self.valida_resposta(resposta)
        return resposta.json()['tribunais_suportados']

    def _tribunal_por_sigla(self, sigla):
        s = sigla.lower()
        for n, t in self.tribunais.items():
            if s == t['sigla']:
                return n
        return None

    def obtem(self, tribunal):
        if not self.tribunais:
            self.tribunais = self._obtem_dados()
        n = self._tribunal_por_sigla(tribunal) or tribunal
        t = self.tribunais.get(n, None)
        if t:
            return Tribunal(n, t)
        raise KeyError('Este tribunal não é suportado')

    def suportado(self, tribunal):
        try:
            self.obtem(tribunal)
        except KeyError:
            return False
        return True


class Tribunal(object):

    def __init__(self, tribunal_numero, tribunal_info):
        self.numero = tribunal_numero
        self.suporta_oab = tribunal_info['suporta_oab']
        self.sigla = tribunal_info['sigla']
        self.estados = tribunal_info['estados']
        self.nome = tribunal_info['nome']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests

from cliente.exceptions import (
    StatusCodeNaoTratado,
    NaoCadastrado,
    NaoCarregado,
    ParametrosIncorretos,
    TribunalNaoSuportado,
)


class BaseAPI(object):
    """Implementa interface básica com a API do SIJ-Andamentos

    Esta classe implementa chamadas à API e tratamento de dados. Esta não é uma
    classe fim e deve ser utilizada apenas através das classes "filha".
    """
    URL_DEFAULT = 'https://andamentos.justicafacil.com.br/'
    PATH_PROCESSO = 'api/v1/processos/'
    PATH_OAB = 'api/v1/oabs/'
    PATH_TRIBUNAL = 'api/v1/tribunais/'

    NaoCadastrado = NaoCadastrado
    NaoCarregado = NaoCarregado

    def __init__(self, token, url=URL_DEFAULT):
        self.token = token
        self.headers = {'Authorization': 'Token ' + token}
        self.url = url if url.endswith('/') else url + '/'

    def conectado(self):
        resposta = self._executa('auth/postback/')
        if resposta.status_code == 200:
            return True
        return False

    def _executa(self, path, method='get', params={}):
        dominio = self.url + path
        dominio = dominio if dominio.endswith('/') else dominio + '/'
        executa = getattr(requests, method)
        resposta = executa(dominio, params, headers=self.headers)
        return resposta

    def executa_processo(self, cnj=None, method='get', params={}):
        path = self.PATH_PROCESSO + cnj + '/' if cnj else self.PATH_PROCESSO
        resposta = self._executa(path, method, params)
        return resposta

    def executa_oab(self, registro=None, method='get', params={}):
        path = self.PATH_OAB + registro + '/' if registro else self.PATH_OAB
        resposta = self._executa(path, method, params)
        return resposta

    def executa_tribunal(self):
        resposta = self._executa(self.PATH_TRIBUNAL, 'get')
        return resposta

    def valida_resposta(self, resposta):
        """Valida um argumento do tipo `requests.resposta` vindo da API

        Valida uma resposta do tipo `requests.resposta` quanto ao status HTTP
        e retorna um booleano "created", indicando se algum recurso foi criado.
        """
        if resposta.status_code == 404:
            raise self.NaoCadastrado()
        elif resposta.status_code == 400:
            raise ParametrosIncorretos()
        elif resposta.status_code == 501:
            raise TribunalNaoSuportado()
        elif resposta.status_code == 201:
            return True
        elif resposta.status_code == 200:
            return False
        raise StatusCodeNaoTratado()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import arrow

from cliente import BaseAPI
from cliente.exceptions import AndamentoSemData


class Adaptador(object):
    """Mixin de métodos para adaptação nas classes compostas

    Os métodos desta classe são utilizados para auxiliar na composição de
    outras classes. Por exemplo, quando um Andamento vai "se preencher" com
    os anexos, basta chamar Anexo.carrega(dados) para que dados se torne uma
    lista de anexos.
    """

    @classmethod
    def carrega(cls, dados):
        lista_dados = []
        for dado in dados:
            lista_dados.append(cls(dado))
        return lista_dados


class Parte(Adaptador):
    """Parte de um Processo. """

    def __init__(self, dados):
        self.nome = dados['nome']
        self.designacao = dados['designacao']
        self.advogados = Advogado.carrega(dados['advogados'])


class Advogado(Adaptador):
    """Advogado de uma Parte. """

    def __init__(self, dados):
        self.nome = dados['nome']
        self.oab = dados['oab']


class Andamento(Adaptador):
    """Andamento de um Processo. """

    def __init__(self, dados):
        self.id = dados['id']
        self.texto = re.sub(r'\s{2,}', ' ', dados['texto'])
        self.complemento = dados['complemento']
        self.anexos = Anexo.carrega(dados['anexos'])
        data = dados.get('data')
        if not data:
            raise AndamentoSemData()
        self.data = arrow.get(data).datetime


class Anexo(Adaptador):
    """Anexo de um andamento. """
    URL_DOWNLOAD = 'http://anexos.sij-andamentos.justicafacil.com.br' \
                   '.s3-website-us-east-1.amazonaws.com/'

    def __init__(self, dados):
        self.id = dados['id']
        self.nome_arquivo = dados['nome_arquivo']
        self.corrompido = dados['corrompido']
        self.meta = dados['meta']

    def url(self):
        """ URL para download do anexo """
        return self.URL_DOWNLOAD + self.nome_arquivo


class Processo(BaseAPI):
    """Encapsula um processo buscado na API do SIJ-Andamentos.

    Todo processo buscado na API é representado por uma entidade desta classe.
    Os atributos desta classe correspondem aos atributos disponíveis na API.

    Args:
        cnj(str): Número CNJ no tipo 0000000-00.0000.0.00.0000
        token(str): Token de acesso à API
        url(str, optional): URL da API.

    Attributes:
        id(int): ID do processo na base de dados do SIJ-Andamentos
        cnj(str): CNJ do processo
        detalhes(dict): Detalhes do processo específicos do tribunal
        assuntos(:obj:`list` of :obj:`str`): Lista de assuntos de um processo
        instancia(int): Instância do processo. Pode ser 1, 2, 3 ou 4.
        status(int): Status do processo. De acordo com
            `cliente.processo.Processo.codigo_status`
        orgao_julgador(str): Orgão julgador do processo
        ultima_atualizacao(datetime.datetime): Última atualização daquele
            processo realizada
        ativo(bool): Define se a instância do processo está ativa
        partes(:obj:`list` of :obj:`cliente.processo.Parte`): Partes de um
            processo
        andamentos(:obj:`list` of :obj:`cliente.processo.Andamento`):
            Andamentos de um processo
    """

    novo = 0
    cadastrado = 1
    cadastro_invalido = 2
    sigiloso = 3

    codigo_status = {
        0: 'novo',
        1: 'cadastrado',
        2: 'cadastro_invalido',
        3: 'sigiloso',
    }

    def __init__(self, cnj, token, url=BaseAPI.URL_DEFAULT):
        super(Processo, self).__init__(token, url)
        self.cnj = cnj
        self.carregado = False
        self._id = None
        self._detalhes = None
        self._assuntos = None
        self._tribunal = None
        self._comarca = None
        self._instancia = None
        self._status = None
        self._orgao_julgador = None
        self._ultima_atualizacao = None
        self._ativo = None
        self._partes = None
        self._andamentos = None

    def cadastra(self):
        """Cadastra processo na API do SIJ-Andamentos

        Inicia-se uma busca pelas instâncias do processo na API. O processo em
        questão será incluído na busca diária por andamentos.

        Note:
            **Todas** as vezes que este método for chamado, o processo será
            buscado em todas as instâncias novamente. Usar este método é
            indicado caso se queira saber se o processo está em uma nova
            instância.
        """
        resposta = self.executa_processo(self.cnj, 'post')
        return self.valida_resposta(resposta)

    def favorita(self):
        """Marca todas as instâncias deste processo (CNJ) como prioritários

        Um processo prioritário é buscado de hora em hora.
        """
        params = {'prioritario': True}
        resposta = self.executa_processo(self.cnj, 'patch', params)
        self.valida_resposta(resposta)

    def desfavorita(self):
        """Marca todas as instâncias deste processo com prioridade normal

        Um processo com prioridade normal é buscado uma vez por dia.
        """
        params = {'prioritario': False}
        resposta = self.executa_processo(self.cnj, 'patch', params)
        self.valida_resposta(resposta)

    def carrega_ativo(self, numero_andamentos=None):
        """Carrega instância ativa do processo

        Um processo pode estar em mais de uma instância. O processo ativo é o
        processo que tem o andamento mais recente.
        """
        params = {'ativo': True}
        if numero_andamentos:
            params['numero_andamentos'] = numero_andamentos
        response = self._busca_processo(params)
        self.preenche(response)
        return self

    def _busca_processo(self, params):
        resposta = self.executa_processo(self.cnj, 'get', params)
        self.valida_resposta(resposta)
        return resposta.json()[0]

    def preenche(self, dados):
        self._id = dados['id']
        self._detalhes = dados['detalhes']
        self._assuntos = dados['assuntos']
        self._instancia = dados['instancia']
        self._tribunal = dados['tribunal']
        self._comarca = dados['comarca']
        self._status = dados['status']
        self._orgao_julgador = dados['orgao_julgador']
        self._ativo = dados['ativo']
        self._partes = Parte.carrega(dados['partes'])
        self._andamentos = Andamento.carrega(dados['andamentos'])
        # Obtem informações da data
        data = arrow.get(dados.get('ultima_atualizacao'))
        self._ultima_atualizacao = data.datetime
        self.carregado = True

    def _check_carregado(self):
        if not self.carregado:
            raise self.NaoCarregado(
                'Utilize Processo.carrega_ativo() ou '
                'Processo.carrega_processos() antes de'
                ' tentar acessar algum atributo.')

    @property
    def id(self):
        self._check_carregado()
        return self._id

    @property
    def detalhes(self):
        self._check_carregado()
        return self._detalhes

    @property
    def assuntos(self):
        self._check_carregado()
        return self._assuntos

    @property
    def instancia(self):
        self._check_carregado()
        return self._instancia

    @property
    def status(self):
        self._check_carregado()
        return self._status

    @property
    def tribunal(self):
        self._check_carregado()
        return self._tribunal

    @property
    def comarca(self):
        self._check_carregado()
        return self._comarca

    @property
    def orgao_julgador(self):
        self._check_carregado()
        return self._orgao_julgador

    @property
    def ativo(self):
        self._check_carregado()
        return self._ativo

    @property
    def partes(self):
        self._check_carregado()
        return self._partes

    @property
    def andamentos(self):
        self._check_carregado()
        return self._andamentos

    @property
    def ultima_atualizacao(self):
        self._check_carregado()
        return self._ultima_atualizacao


class Processos(BaseAPI):
    """Busca por lista de processos com mesmo número CNJ

    Busca processos na API e os armazena em uma lista de tipos
    `cliente.processo.Processo`, de forma que os elementos podem ser acessados
    através do operador de índice.

    Args:
        cnj(`str`): Número CNJ no tipo 0000000-00.0000.0.00.0000
        token(`str`): Token de acesso à API
        url(`str`, optional): URL da API.
    """

    def __init__(self, cnj, token, url=BaseAPI.URL_DEFAULT):
        super(Processos, self).__init__(token, url)
        self.cnj = cnj
        self.carregado = False
        self._lista = []

    def carrega(self, inicio=None, fim=None, numero_andamentos=None):
        """Carrega todas as instâncias de um CNJ para uma lista de
        cliente.processo.Processo

        Quando este método é chamado, é feita uma busca por todas as instâncias
        de um processo, inclusive as inativas, criando uma lista de Processo.
        """
        params = {}
        if inicio:
            params['inicio'] = inicio
        if fim:
            params['fim'] = fim
        if numero_andamentos:
            params['numero_andamentos'] = numero_andamentos
        processos = self._busca_processos(params)
        for proc in processos:
            processo = Processo(self.cnj, self.token, self.url)
            processo.preenche(proc)
            self._lista.append(processo)
        return self

    def _busca_processos(self, params):
        resposta = self.executa_processo(self.cnj, 'get', params)
        self.valida_resposta(resposta)
        return resposta.json()

    def __getitem__(self, key):
        if not isinstance(key, int):
            raise TypeError('Os índices precisam ser inteiros, não strings')
        return self._lista[key]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from requests.exceptions import HTTPError


class NaoCarregado(Exception):
    pass


class TribunalNaoSuportado(HTTPError):
    """O tribunal requisitado não é suportado pelo Andamentos.

    Em um número CNJ

        NNNNNNN-DD.AAAA.J.TR.OOOO

    os números "J" e "TR" representam o tribunal acessado para receber as
    informações do processo. Caso o tribunal não seja suportado, o Andamentos
    retorna um status code HTTP 501, e o Cliente-Andamentos lança então uma
    exceção `TribunalNaoSuportado`.

    Para obter informações dos tribunais suportados, veja `cliente.tribunal`.
    """
    pass


class ParametrosIncorretos(HTTPError):
    """Algum parâmetro está incorreto

    Esta exceção é uma representação do status code HTTP 400 e significa que
    algum dos parâmetros passados não pode ser compreendido pela API.
    """
    pass


class NaoCadastrado(HTTPError):
    """O processo ou advogado requisitados não existe na base de dados do
    Andamentos.

    O Cliente-Andamentos tentou realizar alguma ação que exige um recurso
    pré-cadastrado e este recurso não pode ser encontrado. Este erro é uma
    interface para o status code HTTP 404.
    """
    pass


class StatusCodeNaoTratado(Exception):
    """O status code HTTP retornado pela API não foi tratado. Contate os
    administradores da API.
    """
    pass


class AndamentoSemData(Exception):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from cliente import BaseAPI


class Oab(BaseAPI):
    """Encapsula um registro de advogado buscado na API do SIJ-Andamentos.

    Todo advogado (também denominado neste sistema por Oab) buscado na API é
    representado por uma entidade desta classe.  Os atributos desta classe
    correspondem aos atributos disponíveis na API.

    Args:
        registro(`str`): Registro da OAB do advogado no tipo RS12345
        token(`str`): Token de acesso à API
        url(`str`, optional): URL da API.

    Attributes:
        id(int): ID do advogado na base de dados do SIJ-Andamentos
        uf(str): UF do regitro da OAB
        oab(str): Número do registro da OAB
        nome(str): Nome do advogado
        processos(:obj:`list` of :obj:`str`): Lista de processos relacionados a
            uma OAB
    """

    def __init__(self, registro, token, url=BaseAPI.URL_DEFAULT):
        super(Oab, self).__init__(token, url)
        self.registro = registro
        self.carregado = False
        self._id = None
        self._uf = None
        self._oab = None
        self._nome = None
        self._processos = None

    def cadastra(self, nome):
        """Cadastra um registro de Advogado na API

        Se o registro da OAB ainda não existir na API, um novo advogado é
        cadastrado.
        """
        params = {
            'nome': nome,
            'registro': self.registro,
        }
        resposta = self.executa_oab(method='put', params=params)
        return self.valida_resposta(resposta)

    def carrega(self):
        """Carrega os dados de um registro de Advogado

        Os dados de um advogado são carregados para esta instância de Oab e
        podem ser acessadas pelos seus atributos.
        """
        resposta = self.executa_oab(self.registro, 'get')
        self.valida_resposta(resposta)
        self.preenche(resposta.json())
        return self

    def busca_processos(self):
        """Busca todos os processos de um advogado na API

        Para acessar os processos de um advogado, basta usar o método
        `cliente.oab.Oab.carrega()` novamente e acessar o atributo `processos`.

        Note:
            Entre a chamada por `busca_processos()` e o `carrega()`, deve haver
            um tempo, necessário para que o sistema cadastre os novos
            processos. Esse tempo vai depender da quantidade de processos
            encontrados para o registro OAB buscado.
        """
        resposta = self.executa_oab(self.registro, 'post')
        return self.valida_resposta(resposta)

    def preenche(self, dados):
        self._id = dados['id']
        self._uf = dados['uf']
        self._oab = dados['oab']
        self._nome = dados['nome']
        self._processos = dados['processos']
        self.carregado = True

    def _check_carregado(self):
        if not self.carregado:
            raise self.NaoCarregado(
                'Utilize Oab.carrega() antes de'
                ' tentar acessar algum atributo.')

    @property
    def id(self):
        self._check_carregado()
        return self._id

    @property
    def uf(self):
        self._check_carregado()
        return self._uf

    @property
    def oab(self):
        self._check_carregado()
        return self._oab

    @property
    def nome(self):
        self._check_carregado()
        return self._nome

    @property
    def processos(self):
        self._check_carregado()
        return self._processos



from setuptools import setup
from os import path

BASE_DIR = path.abspath(path.dirname(__file__))


setup(
    name='cliente-andamentos',
    version='0.1.7',
    description='Cliente python para o sistema Andamentos',
    long_description="""
        Cliente Andamentos
        ==================

        O Cliente Andamentos e uma biblioteca que faz interface com a API do
        *Andamentos*.

        # Interface

        O Cliente e formado pelas seguintes classes:
        - BaseAPI
        - Processo
        - Tribunais
        - Parte
        - Andamento
        - Anexo
        - Oab

        Atraves desses objetos e possivel fazer contato com a API de uma forma
        interativa. Um exemplo seria:

        ```python
        from cliente.oab import Oab
        oab = Oab('Joao Doe', 'MG55000', token=algum_token)
        # Salva o advogado na base de dados do Andamentos
        oab.cadastra()
        ```
    """,
    url='https://gitlab.com/sijnet/cliente-andamentos',
    author='Lucas Almeida Aguiar',
    author_email='lucas.tamoios@gmail.com',

    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='api andamentos processos justica',

    packages=['cliente'],

    install_requires=['requests', 'arrow'],

    extras_require={
        'dev': ['mock', 'pytest', 'pytest-cov'],
        'test': ['mock', 'pytest', 'pytest-cov'],
    },

)
